// wuipath.c - implement WUIMAN path object
// Copyright (c) 1994 by Ron Burk

#include <string.h>
#include "wuipath.h"


// TWuiPath - Handle moving up and down the database "path"
TWuiPath::TWuiPath(char *TotalPath)
    :   AbsolutePath_(TotalPath), RelativePath_(TotalPath)
    {   MEMBERASSERT();
    ASSERT(TotalPath != NULL);
    ASSERT(TotalPath[0] == '/');
    }

// Descend() - Move relative path down one node.
int TWuiPath::Descend()
    {   MEMBERASSERT();
    char    *Child = strchr(RelativePath_, '/');
    if(!Child)   // if no children
        return FALSE;
    // ignore trailers (e.g. "/a/b/c/")
    else if(Child[1] == '\0')
        return FALSE;
    // else there's *some* kind of child name
    else
        {
        RelativePath_   = Child+1;
        return TRUE;
        }
    }

// Ascend() - Move relative path up one node.
int TWuiPath::Ascend()
    {   MEMBERASSERT();
    switch(RelativePath_ - AbsolutePath_)
        {
        case    0:
            return FALSE;
        case    1:
            RelativePath_   = AbsolutePath_;
            return TRUE;
        default:
            RelativePath_   -= 2;
            while(RelativePath_ > AbsolutePath_)
                if(*RelativePath_ == '/')
                    break;
                else
                    --RelativePath_;
            ASSERT(*RelativePath_ == '/');
            ++RelativePath_;
            return TRUE;
        }
    }

// CurrentName() - return first name of relative path.
TWuiName    TWuiPath::CurrentName()
    {   MEMBERASSERT();
    char        Name[TWuiName::MAXNAME+1];
    const char *Rover = strchr(RelativePath(), '/');
    int         Length;
    if(Rover)
        Length  = (int)(Rover-RelativePath());
    else
        Length  = strlen(RelativePath());
    const char *Start = RelativePath();
    // AWuiObject::Get/Set needs this
    if(*Start == '.')
        ++Start;
    strncpy(Name, Start, Length);
    Name[Length]    = '\0';
    return TWuiName(Name);
    }

// Push() - Append to the path.
int     TWuiPath::Push(const char *SubPath)
    {   MEMBERASSERT();
    char   *Rover = AbsolutePath_ + strlen(AbsolutePath_);
    if(Rover > AbsolutePath_)
        --Rover;    // point to char before EOS
    if(*Rover != '/')
        strcat(AbsolutePath_, "/");
    strcat(AbsolutePath_, SubPath);
    return TRUE;    // Need to check for overflow!
    }
// Push() - Append to the path (TWuiName version)
int     TWuiPath::Push(const TWuiName SubPath)
    {   MEMBERASSERT();
    char    Name[TWuiName::MAXNAME+1];
    return Push(SubPath.GetName(Name));
    }

// Pop() - Pop the last node off the path.
int TWuiPath::Pop()
    {   MEMBERASSERT();
    // if already at root, can't pop
    if(!strcmp(AbsolutePath_, "/"))
        return FALSE;
    char   *Rover = AbsolutePath_ + strlen(AbsolutePath_);
    if(Rover > AbsolutePath_)
        --Rover;    // point to char before EOS
    if(Rover > AbsolutePath_ && *Rover == '/')
        --Rover;    // skip any bogus trailing '/'
    while(Rover > AbsolutePath_ && *Rover != '/')
        --Rover;
    // if string did not start with "/", this is bad!
    if(Rover < AbsolutePath_)
        return FALSE;
    else    // else Rover points at '/'
        {
        // as always, special case for root
        if(Rover == AbsolutePath_)
            ++Rover;
        *Rover = '\0';
        return TRUE;
        }
    }

int     TWuiPath::IsLastNode() const
    {   MEMBERASSERT();
    return strchr(RelativePath_, '/') == NULL;
    }

void    TWuiPath::CopyRelativePath(char *Output) const
    {   MEMBERASSERT();
    strcpy(Output, AbsolutePath_);
    const char *Rover = strchr(RelativePath_, '/');
    if(Rover == NULL)
        Rover   = RelativePath_ + strlen(RelativePath_);
    if(RelativePath_ > AbsolutePath_)
        Output[Rover-AbsolutePath_] = '\0';
    }



